###################################################################################################
# _blobfields.py
#
# Implementation of classes MyFile, MyImage, etc. (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from OFS.Image import Image, File
from cStringIO import StringIO
import string
import time
# Product Imports.
from _globals import *
import _fileutil
import _mimetypes


#--------------------------------------------------------------------------------------------------
# createBlobField
#--------------------------------------------------------------------------------------------------
def createBlobField(self, datatype, file='',lang=None):
  blob = None
  i = MyImage(id='',title='',file='')
  f = MyFile(id='',title='',file='')
  if datatype == DT_IMAGE or datatype == str(i):
    blob = i
  elif datatype == DT_FILE or datatype == str(f):
    blob = f
  if blob is not None:
    blob.aq_parent = self
    blob.lang = lang
    blob.mediadbfile = None
    if type(file) is type({}):
      blob.uploadFile(StringIO(file.get('data','')),file.get('filename',''))
    elif file:
      blob.uploadFile(file,file.filename)
  return blob

#--------------------------------------------------------------------------------------------------
# getLangFilename
#--------------------------------------------------------------------------------------------------
def getLangFilename(filename, lang):
  i = string.rfind(filename,'.')
  name = filename[:i]
  ext = filename[i+1:]
  if lang is not None:
    suffix = '_' + lang
    if len(name) > len(suffix) and name[-len(suffix):] != suffix:
      name = name + suffix
  return name + '.' + ext

#--------------------------------------------------------------------------------------------------
# synchronizeBlobFields
#--------------------------------------------------------------------------------------------------
def synchronizeBlobFields(parent):
  #++ if debug: print "[%s.synchronizeBlobFields]:"%(parent.meta_type)

  # Delete DTML Methods.
  title = 'blob_html'
  ids = []
  for ob in parent.objectValues(['DTML Method']):
    if ob.title == title:
      ids.append(ob.id())
  if len(ids) > 0:
    try:
      #++ if debug: print " * Delete DTML Methods %s..."%str(ids)
      parent.manage_delObjects(ids)
    except:
      #++ if debug: print " * Exception when deleting DTML Methods %s!"%str(ids)
      pass

  # Create DTML Methods.
  for key in parent.getObjAttrs().keys():
    obj_attr = parent.getObjAttr(key)
    datatype = parent.getObjDatatypeKey(obj_attr)
    if datatype in DT_BLOBS:
      for s_lang in parent.getLanguages():
        for s_preview in ['','preview']:
          req = {'lang':s_lang,'preview':s_preview}
          v = parent._getObjAttrValue(obj_attr,datatype,req)
          if v is not None:
          
            # DTML call returns blob_html.
            raw = ''
  	    raw = raw + '<dtml-call \"REQUEST.set(\'lang\',\'%s\')\">\n'%s_lang
  	    raw = raw + '<dtml-return "blob_html(\'%s\',REQUEST,RESPONSE)">\n'%key
  	    
            # Add DTML Method with language suffix.
            filename = v.filename
            for ch in ['%','/','\\',' ','!','#','"']:
              filename = string.replace(filename,ch,'')
            try:
              #++ if debug: print " * Add DTML Method <%s>."%filename
              parent.manage_addDTMLMethod(filename,title,raw)
            except:
              #++ if debug: print " * Exception when adding DTML Method <%s>!"%filename
              pass
              
            # Add DTML Method with language suffix.
            if len(parent.getLanguages()) > 1:
              lang_filename = getLangFilename(filename,s_lang)
              if lang_filename != filename:
                try:
                  #++ if debug: print " * Add DTML Method with language suffix <%s>."%lang_filename
                  parent.manage_addDTMLMethod(lang_filename,title,raw)
                except:
                  #++ if debug: print " * Exception when adding DTML Method with language suffix <%s>!"%lang_filename
                  pass

###################################################################################################

class MyBlob:

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.download: 
    # ---------------------------------------------------------------------------------------------
    def download(self, folder):
      filename = _fileutil.getOSPath('%s/%s'%(folder,self.filename))
      _fileutil.exportObj(self,filename)
      return filename

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.upload:
    # ---------------------------------------------------------------------------------------------
    def upload(self, folder):
      sFilename = "%s/%s"%(folder,self.filename)
      sFilename = _fileutil.getOSPath(sFilename)
      # Backup properties (otherwise manage_upload sets it).
      bk = {}
      for key in self.__xml_attrs__:
        if hasattr(self,key):
          bk[key] = getattr(self,key)
      # Read file to ZODB.
      f = open(sFilename,'rb')
      self.uploadFile(f.read(),self.filename)
      f.close()
      # Restore properties.
      for key in self.__xml_attrs__:
        if bk.has_key(key) and len(str(bk[key])) > 0 and bk[key] not in ['text/x-unknown-content-type']:
          setattr(self,key,bk[key])

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.uploadFile:
    # ---------------------------------------------------------------------------------------------
    def uploadFile(self, data, filename):
      parent = self.aq_parent
      mediadb = parent.getMediaDb()
      if data:
        filename = _fileutil.extractFileName(filename)
        filename = getLangFilename(filename,getattr(self,'lang',None))
        self.filename = filename
        self.uploadData(data)
        if mediadb is not None:
          self.mediadbfile = mediadb.storeFile(self)
          self.data = ''
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getData: 
    # ---------------------------------------------------------------------------------------------
    def getData(self):
      parent = self.aq_parent
      mediadb = parent.getMediaDb()
      mediadbfile = getattr(self,'mediadbfile',None)
      if mediadb is not None and mediadbfile is not None:
        try:
          data = mediadb.retrieveFile(mediadbfile)
        except:
          data = ''
      else:
        data = getattr(self,'data','')
      return data
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.blob_html: 
    # ---------------------------------------------------------------------------------------------
    def blob_html(self, REQUEST, RESPONSE):
      parent = self.aq_parent
      filename = getattr(self,'filename','')
      if len(parent.getLanguages()) > 1:
        getLangFilename(filename,REQUEST.get('lang',None))
      content_type = getattr(self,'content_type','Unknown')
      RESPONSE.setHeader('Cache-Control','public, max-age=3600')
      RESPONSE.setHeader('Content-Type',content_type)
      RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
      return self.getData()
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.src: 
    # 
    #   DEPRECATED!
    # ---------------------------------------------------------------------------------------------
    src__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def src(self, parent, key, REQUEST):
      qs = ''      
      ids = parent.objectIds(['DTML Method'])
      filename = self.getFilename()
      if len(parent.getLanguages()) > 1:
        lang_filename = getLangFilename(filename,REQUEST['lang'])
        if lang_filename in ids:
          filename = lang_filename
      if not filename in ids:
        filename = 'blob_html'
        qs = qs_append(qs,'key',key)
        qs = qs_append(qs,'lang',REQUEST['lang'])
      if isPreviewRequest(REQUEST):
        qs = qs_append(qs,'preview','preview')
      return '%s/%s%s'%(parent.absolute_url(),filename,qs)

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getHref:
    # ---------------------------------------------------------------------------------------------
    getHref__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getHref(self, REQUEST):
      parent = self.aq_parent
      key = self.key
      return self.src(parent,key,REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getFilename: 
    #
    #	Returns filename.
    # ---------------------------------------------------------------------------------------------
    getFilename__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getFilename(self):
      filename = _fileutil.extractFileName(self.filename) 
      filename = string.replace(filename,'%','')
      filename = string.replace(filename,' ','')
      return filename
    
    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.xmlGetTagName: 
    #
    #	Returns <XML> Tag-Name.
    # ---------------------------------------------------------------------------------------------
    def xmlGetTagName(self):
      return 'data'

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getDataSizeStr: 
    #
    #	Display string for file-size.
    # ---------------------------------------------------------------------------------------------
    getDataSizeStr__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getDataSizeStr(self):
      s = ''
      data = self.getData()
      if data is not None:
        s = _fileutil.getDataSizeStr(float(len(data)))
      return s

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getContentType:
    #
    #	MIME-type.
    # ---------------------------------------------------------------------------------------------
    getContentType__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getContentType(self):
      return self.content_type

    # ---------------------------------------------------------------------------------------------
    # 	MyBlob.getMimeTypeIconSrc:
    #
    #	Image source for MIME-type.
    # ---------------------------------------------------------------------------------------------
    getMimeTypeIconSrc__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getMimeTypeIconSrc(self):
      content_type = self.getContentType()
      if not _mimetypes.dctMimeType.has_key(content_type):
        content_type = 'content/unknown'
      return '%s%s'%(self.aq_parent.MISC_ZMS,_mimetypes.dctMimeType[content_type])

###################################################################################################

class MyImage(MyBlob,Image):

    __obj_attrs__ = ['size','width','height','content_type','data','filename','mediadbfile']
    __xml_attrs__ = ['width','height']
    
    def __str__(self):
      return '{{MyImage}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.uploadData:
    # ---------------------------------------------------------------------------------------------
    def uploadData(self, data):
      ob = Image(id='',title='',file='')
      ob.manage_upload(data)
      for key in ['size','width','height','content_type','data']:
        if hasattr(ob,key):
          setattr(self,key,getattr(ob,key))
    
    # ---------------------------------------------------------------------------------------------
    # 	MyImage._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self):
      ob = self
      oClone = MyImage(id='',title='',file='')
      attrs = self.__obj_attrs__
      for attr in attrs:
        if hasattr(ob,attr):
          setattr(oClone,attr,getattr(ob,attr))
      return oClone

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender):
      sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url())+1:]+'/'
      sFilename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
      xml = '\n<%s'%self.xmlGetTagName()
      xml = xml + ' width="%s"'%str(getattr(self,'width',''))
      xml = xml + ' height="%s"'%str(getattr(self,'height',''))
      xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
      xml = xml + ' filename="%s%s"'%(sFilepath,sFilename)
      xml = xml + '>'
      xml = xml + '</%s>'%self.xmlGetTagName()
      return xml

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.getWidth:
    # ---------------------------------------------------------------------------------------------
    getWidth__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getWidth(self):
      return self.width

    # ---------------------------------------------------------------------------------------------
    # 	MyImage.getHeight:
    # ---------------------------------------------------------------------------------------------
    getHeight__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
    def getHeight(self):
      return self.height

###################################################################################################

class MyFile(MyBlob,File):

    __obj_attrs__ = ['size','content_type','data','filename','mediadbfile']
    __xml_attrs__ = []

    def __str__(self):
      return '{{MyFile}}'

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.uploadData:
    # ---------------------------------------------------------------------------------------------
    def uploadData(self, data):
      ob = File(id='',title='',file=data)
      for key in ['content_type','data']:
        if hasattr(ob,key):
          setattr(self,key,getattr(ob,key))
    
    # ---------------------------------------------------------------------------------------------
    # 	MyFile._getCopy:
    # ---------------------------------------------------------------------------------------------
    def _getCopy(self):
      ob = self
      oClone = MyFile(id='',title='',file='')
      attrs = self.__obj_attrs__
      for attr in attrs:
        if hasattr(ob,attr):
          setattr(oClone,attr,getattr(ob,attr))
      return oClone

    # ---------------------------------------------------------------------------------------------
    # 	MyFile.toXml:
    # ---------------------------------------------------------------------------------------------
    def toXml(self, sender):
      sFilepath = sender.absolute_url()[len(sender.getDocumentElement().absolute_url())+1:]+'/'
      sFilename = _fileutil.getOSPath(_fileutil.extractFileName(getattr(self,'filename','')))
      xml = '\n<%s'%self.xmlGetTagName()
      xml = xml + ' content_type="%s"'%str(getattr(self,'content_type',''))
      xml = xml + ' filename="%s%s"'%(sFilepath,sFilename)
      xml = xml + '>'
      xml = xml + '</%s>'%self.xmlGetTagName()
      return xml

###################################################################################################
