###############################################################################################
# _xmllib.py
#
# Implementation of XML-Library (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###############################################################################################

# Imports.
from App.Common import package_home
from Shared.DC.xml import pyexpat
import Globals
import copy
import os
import string
import tempfile
# Product Imports.
from _objattrs import *
from _blobfields import createBlobField
import _globals
import _fileutil


"""
###############################################################################################
#
#   Q U O T E
#
###############################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  xml_quote_chars:
# ---------------------------------------------------------------------------------------------
def xml_quote_chars():
  chs = [ord('&'),ord('"'),ord('<'),ord('>'),ord('?')]
  for ch in range(0,255):
    if ch < 32 or ch > 126:
      chs.append(ch)
  return chs

# ---------------------------------------------------------------------------------------------
#  xml_quote:
# ---------------------------------------------------------------------------------------------
def xml_quote(s):
  return unicode(s,'latin-1').encode('iso-8859-1')

# ---------------------------------------------------------------------------------------------
#  inv_xml_quote:
# ---------------------------------------------------------------------------------------------
def inv_xml_quote(s):
  if type(s) is type(''):
    s = unicode(s,'utf-8').encode('latin-1')
    # Backwards compatibility.
    i = s.find('&#')
    j = s.find(';',i)
    while i >= 0 and j >= 0:
      v = s[i:j+1]
      if v[2] == 'x':
        ch = chr(int(v[3:-1],16))
      else:
        ch = chr(int(v[2:-1]))
      s = string.replace(s,v,ch)
      i = s.find('&#') 
      j = s.find(';',i)
  return s



"""
###############################################################################################
#
#   I M P O R T
#
###############################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  xmlOnCharacterData:
# ---------------------------------------------------------------------------------------------
def xmlOnCharacterData(self, sData, bInCData):
  #++ _globals.myDebugger.write("[%s.xmlOnCharacterData]: sData=%s"%(self.meta_type,sData))
  #-- TAG-STACK
  if self.dTagStack.size() > 0:
    tag = self.dTagStack.pop()
    tag['cdata'] = tag['cdata'] + sData
    self.dTagStack.push(tag)
  pass


# ---------------------------------------------------------------------------------------------
#  xmlOnUnknownStartTag:
# ---------------------------------------------------------------------------------------------
def xmlOnUnknownStartTag(self, sTagName, dTagAttrs):
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownStartTag]: sTagName=%s"%(self.meta_type,sTagName))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownStartTag]: dTagStack[%i]=%s"%(self.meta_type,self.dTagStack.size(),self.dTagStack.toString()))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownStartTag]: dValueStack[%i]=%s"%(self.meta_type,self.dValueStack.size(),self.dValueStack.toString()))
        
  #-- TAG-STACK
  tag = {'name':sTagName,'attrs':dTagAttrs,'cdata':''}
  tag['dValueStack'] = self.dValueStack.size()
  self.dTagStack.push(tag)
  
  #-- VALUE-STACK
  if sTagName == 'lang':
    self.dValueStack.push({})
  elif sTagName == 'dictionary':
    self.dValueStack.push({})
  elif sTagName == 'list':
    self.dValueStack.push([])
    
  #-- Return
  return 1  # accept any unknown tag
        
        
# ---------------------------------------------------------------------------------------------
#  xmlOnUnknownEndTag:
# ---------------------------------------------------------------------------------------------
def xmlOnUnknownEndTag(self, sTagName):
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: sTagName=%s"%(self.meta_type,sTagName))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: dTagStack[%i]=%s"%(self.meta_type,self.dTagStack.size(),self.dTagStack.toString()))
  #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: dValueStack[%i]=%s"%(self.meta_type,self.dValueStack.size(),self.dValueStack.toString()))
        
  #-- TAG-STACK
  tag = self.dTagStack.pop()        
  name = tag['name']
  if name==sTagName:

      attrs = tag['attrs']
      cdata = inv_xml_quote(tag['cdata'])
            
      #-- ITEM (DICTIONARY|LIST) --
      #----------------------------
      if sTagName in ['list','dictionary']:
        pass
      elif sTagName in ['item']:
        if tag['dValueStack'] < self.dValueStack.size():
          item = self.dValueStack.pop()
        else:
          item = cdata
        value = self.dValueStack.pop()
        if type(value) is type({}):
          key = attrs[attrs.index('key')+1]
          value[key] = item
        if type(value) is type([]):
          value.append(item)
        self.dValueStack.push(value)
              
      #-- DATA (IMAGE|FILE) --
      #-----------------------
      elif sTagName in ['data']:
        value = {}
        for attr_key in ['width','height','content_type','filename']:
          if attr_key in attrs:
            item = attrs[attrs.index(attr_key)+1]
            value[attr_key] = item
        self.dValueStack.push(value)
            
      #-- LANGUAGE --
      #--------------
      elif sTagName in ['lang']:
        lang = self.getPrimaryLanguage()
        if 'id' in attrs:
          lang = attrs[attrs.index('id')+1]
        if self.dValueStack.size() > 1:
          item = self.dValueStack.pop()
        else:
          item = cdata
        value = _globals.nvl(self.dValueStack.pop(),{})
        value[lang] = item
        self.dValueStack.push(value)
        ##### Object State ####
        req = {}
        req['lang'] = lang
        req['AUTHENTICATED_USER'] = 'xml'
        self.setObjStateNew(req)
              
      #-- OBJECT-ATTRIBUTES --
      #-----------------------
      elif sTagName in self.getObjAttrs().keys():
      
      	obj_attr = self.getObjAttr(sTagName)
      	
        #-- DATATYPE
        datatype = self.getObjDatatypeKey(obj_attr)
        
        #-- Multi-Language Attributes.
        if isLangAttr(obj_attr):
          item = self.dValueStack.pop()
          if item is not None:
            for s_lang in item.keys():
              value = item[s_lang]
              # Data
              if datatype in _globals.DT_BLOBS:
                if type(value) is type({}) and len(value.keys()) > 0:
                  ob = createBlobField(self,datatype,"",s_lang)
                  for attr_key in value.keys():
                    if _globals.debug: print ob,attr_key,value[attr_key]
                    setattr(ob,attr_key,value[attr_key])
                  self.__initObjProperty__(sTagName,ob,s_lang)
              # Others
              else:
                self.__initObjProperty__(sTagName,value,s_lang)
                #++ _globals.myDebugger.write("[%s.xmlOnUnknownEndTag]: %s_%s=%s"%(self.meta_type,sTagName,s_lang,str(value)))
              
        #-- Complex Attributes (Dictionary|List).
        elif self.dValueStack.size() > 0:
          value = self.dValueStack.pop()
          # Data
          if datatype in _globals.DT_BLOBS:
            if type(value) is type({}) and len(value.keys()) > 0:
              ob = createBlobField(self,datatype,"",None)
              for attr_key in value.keys():
                if _globals.debug: print ob,attr_key,value[attr_key]
                setattr(ob,attr_key,value[attr_key])
              self.__initObjProperty__(sTagName,ob,None)
          # Others
          else:
            self.__initObjProperty__(sTagName,value)
          if self.dValueStack.size() > 0:
            raise "Items on self.dValueStack=%s"%self.dValueStack.toString()
        
        #-- Simple Attributes (String, Integer, etc.)
        else:
          value = cdata
          #-- OPTIONS
          if obj_attr.has_key('options'):
            options = obj_attr['options']
            if type(options) is type([]):
              try:
                i = options.index(int(value))
                if i%2==1: value = options[i-1]
              except:
                try: 
                  i = options.index(str(value))
                  if i%2==1: value = options[i-1]
                except: 
                  pass
          self.__initObjProperty__(sTagName,value)

        # Clear value stack.
        # self.dValueStack.clear()
      
      #-- OTHERS --
      #------------
      else:
        value = self.dTagStack.pop()
        cdata = value.get('cdata','')
        cdata = cdata + '<' + tag['name'] 
        for i in range(0,len(tag['attrs'])/2):
          attr_name = tag['attrs'][i*2]
          attr_value = tag['attrs'][i*2+1]
          cdata = cdata + ' ' + attr_name + '="' + attr_value + '"'
        cdata = cdata + '>' + tag['cdata'] 
        cdata = cdata + '</' + tag['name'] + '>'
        value['cdata'] = cdata
        self.dTagStack.push(value)

      return 1 # accept matching end tag
  else:
      return 0  # don't accept any unknown tag


"""
###############################################################################################
#
#   E X P O R T
#
###############################################################################################
"""

# ---------------------------------------------------------------------------------------------
#  toCdata
# ---------------------------------------------------------------------------------------------
def toCdata(self, s, cdata=0):

  # HTML TIDY
  path = self.getConfProperty('InstalledProducts.tidy','')
  if cdata == 0 and len(path) > 0:

    # Create temporary folder.
    folder = tempfile.mktemp()
    os.mkdir(folder)
      
    # Save to temporary file.
    xhtmlfilename = _fileutil.getOSPath('%s/xhtml.html'%folder)
    _fileutil.exportObj(s,xhtmlfilename)

    # Call HTML TIDY.
    conf = _fileutil.getOSPath(package_home(globals())+'/conf/xsl/tidy.html2xhtml.conf')
    cmd = '%s -asxml -config %s -m %s'%(path,conf,xhtmlfilename)
    os.system(cmd)
    
    # Read from temporary file.
    f = open(xhtmlfilename,'rb')
    rtn = xml_quote(f.read())
    rtn = string.strip(rtn)
    prefix = '<p>'
    if s[:len(prefix)] != prefix and rtn[:len(prefix)] == prefix:
      rtn = rtn[len(prefix):]
      suffix = '</p>'
      if s[-len(suffix):] != suffix and rtn[-len(suffix):] == suffix:
        rtn = rtn[:-len(suffix)]
    f.close()

    # Remove temporary files.
    _fileutil.remove(folder,deep=1)
    
  else:
    rtn = '<![CDATA[%s]]>'%xml_quote(s)
    
  return rtn


# ---------------------------------------------------------------------------------------------
#  toXml:
# ---------------------------------------------------------------------------------------------
def toXml(self, value, cdata=0):
  xml = ''
  
  if value is not None:
    
    # Dictionaries
    if type(value) is type({}):
      keys = value.keys()
      if len(keys) > 0:
        xml = xml + '\n<dictionary>' 
        keys.sort()
        for key in keys:
          xml = xml + '\n<item key="%s">%s</item>'%(key,toXml(self,value[key],cdata))
        xml = xml + '</dictionary>'
    
    # Lists
    elif type(value) is type([]):
      if len(value) > 0:
        xml = xml + '\n<list>'
        for item in value:
          xml = xml + '\n<item>%s</item>'%toXml(self,item,cdata)
        xml = xml + '</list>'
    
    # Others.
    else:
      s_value = str(value)
      if len(s_value) > 0:
        xml = xml + toCdata(self,s_value,cdata)
  
  # Return xml.
  return xml


# ---------------------------------------------------------------------------------------------
#  getAttrToXml:
# ---------------------------------------------------------------------------------------------
def getAttrToXml(self, obj_attr, REQUEST={}):
  xml = ''
  
  #-- DATATYPE
  datatype = self.getObjDatatypeKey(obj_attr)
  
  #-- VALUE
  value = self._getObjAttrValue(obj_attr,datatype,REQUEST)
  
  if value is not None:
    
    # Retrieve value from options.
    if obj_attr.has_key('options'):
      options = obj_attr['options']
      try:
        i = options.index(int(value))
        if i%2==0: value = options[i+1]
      except:
        try:
          i = options.index(str(value))
          if i%2==0: value = options[i+1]
        except:
          pass
    
    # Objects.
    if datatype in _globals.DT_BLOBS:
      xml = xml + value.toXml(self)
    
    # Dates.
    elif datatype in _globals.DT_DATETIMES:
      xml = xml + toXml(self,self.getLangFmtDate(value,self.getPrimaryLanguage(),'DATE_FMT'))
    
    # Others.
    else:
      xml = xml + toXml(self,value)
    
  # Return xml.
  return xml

# ---------------------------------------------------------------------------------------------
#  getObjPropertyToXml:
# ---------------------------------------------------------------------------------------------
def getObjPropertyToXml(self, obj_attr):
  xml = ''
  # Multi-Language Attributes.
  if isLangAttr(obj_attr):
    for s_lang in self.getLanguages():
      req = {'lang':s_lang}
      s_attr_xml = getAttrToXml(self,obj_attr,req)
      if len(s_attr_xml) > 0:
        xml = xml + '\n<lang'
        if len(self.getLanguages())>1:
          xml = xml + ' id="%s"'%s_lang
        xml = xml + '>%s</lang>'%s_attr_xml
  # Simple Attributes.
  else:
    xml = xml + getAttrToXml(self,obj_attr)
  # Return xml.
  return xml

# ---------------------------------------------------------------------------------------------
#  getObjToXml:
# ---------------------------------------------------------------------------------------------
def getObjToXml(self):
  if _globals.debug: print "[%s.getObjToXml]:"%(self.meta_type)
  xml = ''
  # Start tag.
  xml = xml + '\n<%s'%self.xmlGetTagName()
  xml = xml + '\n id="%s"'%self.id
  if self.getParentNode() is not None and self.getParentNode().meta_type == 'ZMSCustom':
    xml = xml + '\n id_prefix="%s"'%_globals.id_prefix(self.id)
  if self.meta_type == 'ZMSCustom':
    xml = xml + '\n meta_id="%s"'%self.meta_id
  xml = xml + '>'
  # Attributes.
  for key in self.getObjAttrs().keys():
    obj_attr = self.getObjAttr(key)
    if obj_attr['xml']:
      s_obj_prop_xml = getObjPropertyToXml(self,obj_attr)
      if len(s_obj_prop_xml) > 0:
        xml = xml + '\n<%s>%s</%s>'%(key,s_obj_prop_xml,key)
  # Process children.
  if len(self.objectValues()) > 0:
    for child in self.getChildNodes():
      xml = xml + getObjToXml(child)
  # End tag.
  xml = xml + '</%s>'%self.xmlGetTagName()
  # Return xml.
  return xml


"""
###############################################################################################
#
#   P A R S E R
#
###############################################################################################
"""
class XmlAttrBuilder:
    "class XmlAttrBuilder"

    ######## class variables ########
    iBufferSize=1028 * 32   # buffer size for XML file parsing

    ###############################################################################################
    # Builder.__init__(self):
    #
    # Constructor.
    ###############################################################################################
    def __init__(self):
        """ Builder.__init__ """
        pass


    ###############################################################################################
    # Builder.parse(self, input):
    #
    # Parse a given XML document.
    #
    # IN:  input = XML document as string
    #            = XML document as file object   
    # OUT: value or None, if nothing was parsed
    ###############################################################################################
    def parse(self, input):
        """ Builder.parse """
        
        # prepare builder
        self.dValueStack = _globals.MyStack()
        self.dTagStack   = _globals.MyStack()
        
        # create parser object
        p = pyexpat.ParserCreate()
        
        # connect parser object with handler methods
        p.StartElementHandler = self.OnStartElement
        p.EndElementHandler = self.OnEndElement
        p.CharacterDataHandler = self.OnCharacterData
        p.StartCdataSectionHandler = self.OnStartCData
        p.EndCdataSectionHandler = self.OnEndCData
        p.ProcessingInstructionHandler = self.OnProcessingInstruction
        p.CommentHandler = self.OnComment
        p.StartNamespaceDeclHandler = self.OnStartNamespaceDecl
        p.EndNamespaceDeclHandler = self.OnEndNamespaceDecl
        
        #### parsing ####
        if _globals.debug: print "#### parsing ####"
        if type(input)=='string':
          # input is a string!
          rv = p.Parse(input, 1)
        else:
          # input is a file object!
          while 1:
            if Globals.DatabaseVersion == '3':
              get_transaction().commit(1)
            
            v=input.read(self.iBufferSize)
            if v=="":
              rv = 1
              break
            
            rv = p.Parse(v, 0)
            if not rv:
              break 
              
        # raise parser exception           
        if not rv:
            raise ParseError('%s at line %s' % (pyexpat.ErrorString(p.ErrorCode), p.ErrorLineNumber))
        
        return self.dValueStack.pop()


    ###############################################################################################
    # Builder.OnStartElement(self, name, attrs):
    #
    # Handler of XML-Parser: 
    # Called at the start of a XML element (resp. on occurence of a XML start tag).
    # Usually, the occurence of a XML tag induces the instanciation of a new node object. Therefore,
    # Builder contains a mapping table ("dGlobalAttrs"), that maps XML tags to python classes. The
    # newly created node object is then made current. If no matching class is found for a XML tag,
    # the event handler "xmlOnUnknownStart()" is called on the current object.
    #
    # IN: name  = element name (=tag name)
    #     attrs = dictionary of element attributes
    ###############################################################################################
    def OnStartElement(self, sTagName, dTagAttrs):
        """ Builder.OnStartElement """
        if _globals.debug: print "OnStartElement(" + str(sTagName) + "," + str(dTagAttrs) + ") "
        
        #-- TAG-STACK
        tag = {'name':sTagName,'attrs':dTagAttrs,'cdata':''}
        tag['dValueStack'] = self.dValueStack.size()
        self.dTagStack.push(tag)
        
        #-- VALUE-STACK
        if sTagName == 'dictionary':
          self.dValueStack.push({})
        elif sTagName == 'list':
          self.dValueStack.push([])


    ###############################################################################################
    # Builder.OnEndElement(self, name):
    #
    # Handler of XML-Parser: 
    # Called at the end of a XML element (resp. on occurence of a XML end tag).
    #
    # IN: name  = element name (=tag name)
    ###############################################################################################
    def OnEndElement(self, sTagName):
        """ Builder.OnEndElement """
        if _globals.debug: print "OnEndElement(" + str(sTagName) + ")"
        
        #-- TAG-STACK
        tag = self.dTagStack.pop()        
        name = tag['name']
        attrs = tag['attrs']
        cdata = inv_xml_quote(tag['cdata'])
        
        if name != sTagName:
          raise ParseError("Unmatching end tag (" + sTagName + ")")
        
        #-- ITEM
        if sTagName in ['item']:
          if tag['dValueStack'] < self.dValueStack.size():
            item = self.dValueStack.pop()
          else:
            item = cdata
          value = self.dValueStack.pop()
          if type(value) is type({}):
            key = attrs[attrs.index('key')+1]
            value[key] = item
          if type(value) is type([]):
            value.append(item)
          self.dValueStack.push(value)


    ###############################################################################################
    # Builder.OnCharacterData(self, data):
    #
    # Handler of XML-Parser:
    # Called after plain character data was parsed. Forwards the character data to the current 
    # node. The class attribute "bInCData" determines, wether the character data is nested in a 
    # CDATA block.
    #
    # IN: data = character data string
    ###############################################################################################
    def OnCharacterData(self, sData):
        """ Builder.OnCharacterData """
        if _globals.debug: print "OnCharacterData(" + str(sData) + ")"
        
        #-- TAG-STACK
        if self.dTagStack.size() > 0:
          tag = self.dTagStack.pop()
          tag['cdata'] = tag['cdata'] + sData
          try:
            tag['cdata'] = int(tag['cdata'])
          except:
            pass
          self.dTagStack.push(tag)


    ###############################################################################################
    # Builder.OnStartCData(self):
    #
    # Handler of XML-Parser:
    # Called at the start of a CDATA block (resp. on occurence of the "CDATA[" tag).
    ###############################################################################################
    def OnStartCData(self):
        """ Builder.OnStartCData """
        self.bInCData=_globals.true


    ###############################################################################################
    # Builder.OnEndCData(self):
    #
    # Handler of XML-Parser:
    # Called at the end of a CDATA block (resp. on occurence of the "]" tag).
    ###############################################################################################
    def OnEndCData(self):
        """ Builder.OnEndCData """
        self.bInCData=_globals.false


    ###############################################################################################
    # Builder.OnProcessingInstruction(self, target, data):
    #
    # Handler of XML-Parser:
    # Called on occurence of a processing instruction.
    #
    # IN: target = target (processing instruction)
    #     data   = dictionary of data
    ###############################################################################################
    def OnProcessingInstruction(self, target, data):
        """ Builder.OnProcessingInstruction """
        pass  # ignored


    ###############################################################################################
    # Builder.OnComment(self, data):
    #
    # Handler of XML-Parser:
    # Called on occurence of a comment.
    #
    # IN: data = comment string
    ###############################################################################################
    def OnComment(self, data):
        """ Builder.OnComment """
        pass  # ignored


    ###############################################################################################
    # Builder.OnStartNamespaceDecl(self, prefix, uri):
    #
    # Handler of XML-Parser:
    # Called at the start of a namespace declaration.
    #
    # IN: prefix = prefix of namespace
    #     uri    = namespace identifier
    ###############################################################################################
    def OnStartNamespaceDecl(self, prefix, uri):
        """ Builder.OnStartNamespaceDecl """
        pass  # ignored


    ###############################################################################################
    # Builder.OnEndNamespaceDecl(self, prefix):
    #
    # Handler of XML-Parser:
    # Called at the end of a namespace declaration.
    #
    # IN: prefix = prefix of namespace
    ###############################################################################################
    def OnEndNamespaceDecl(self, prefix):
        """ Builder.OnEndNamespaceDecl """
        pass  # ignored

###################################################################################################